<?php
/**
 * Class that manages customer creation and custom updates.
 * @since 3.0.0
 * @package Stripe/Classes
 * @author PaymentPlugins
 *
 */
class WC_Stripe_Customer_Manager {

	public function __construct() {
		add_action ( 'woocommerce_checkout_update_customer', array( 
				$this, 'checkout_update_customer' 
		), 10, 2 );
		add_action ( 'wp_loaded', array( $this, 
				'wp_loaded' 
		) );
	}

	/**
	 *
	 * @param WC_Customer $customer        	
	 * @param array $data        	
	 */
	public function checkout_update_customer($customer, $data) {
		if ($this->user_has_id ( $customer )) {
			if ($this->should_update_customer ( $customer )) {
				$result = $this->update_customer ( $customer );
				if (is_wp_error ( $result )) {
					wc_add_notice ( __ ( 'Error saving customer. Reason: %s', 'woo-stripe-payment' ), $customer_id->get_error_message () );
				}
			}
		} else {
			// create the customer
			$gateway = new WC_Stripe_Gateway ();
			$customer_id = $this->create_customer ( $customer );
			if (! is_wp_error ( $customer_id )) {
				wc_stripe_save_customer ( $customer_id, $customer->get_id () );
			} else {
				wc_add_notice ( __ ( 'Error saving customer. Reason: %s', 'woo-stripe-payment' ), $customer_id->get_error_message () );
			}
		}
	}

	/**
	 *
	 * @param WC_Customer $customer        	
	 */
	private function create_customer($customer) {
		$gateway = new WC_Stripe_Gateway ();
		return $gateway->create_customer ( apply_filters ( 'wc_stripe_customer_args', array( 
				'email' => $customer->get_email (), 
				'name' => sprintf ( '%s %s', $customer->get_first_name (), $customer->get_last_name () ), 
				'phone' => $customer->get_billing_phone (), 
				'metadata' => array( 
						'user_id' => $customer->get_id (), 
						'username' => $customer->get_username (), 
						'website' => get_site_url () 
				) 
		) ) );
	}

	/**
	 *
	 * @param WC_Customer $customer        	
	 */
	private function update_customer($customer) {
		$gateway = new WC_Stripe_Gateway ();
		return $gateway->update_customer ( wc_stripe_get_customer_id ( $customer->get_id () ), array( 
				'email' => $customer->get_email (), 
				'name' => sprintf ( '%s %s', $customer->get_first_name (), $customer->get_last_name () ), 
				'phone' => $customer->get_billing_phone () 
		) );
	}

	/**
	 * Check if the Stripe customer has been created for the WC user.
	 * If there is no Stripe customer then
	 * create one and save it to the WC user.
	 */
	public function wp_loaded() {
		$customer = WC ()->customer;
		if ($customer && $customer->get_id ()) {
			if (! $this->user_has_id ( $customer )) {
				$customer_id = $this->create_customer ( $customer );
				if (! is_wp_error ( $customer_id )) {
					wc_stripe_save_customer ( $customer_id, $customer->get_id () );
				}
			}
		}
	}

	/**
	 *
	 * @param WC_Customer $customer        	
	 */
	private function user_has_id($customer) {
		$id = wc_stripe_get_customer_id ( $customer->get_id () );
		return ! empty ( $id );
	}

	/**
	 * Returns true if the customer should be updated in Stripe.
	 *
	 * @param WC_Customer $customer        	
	 * @return bool
	 */
	private function should_update_customer($customer) {
		$changes = $customer->get_changes ();
		if (! empty ( $changes[ 'billing' ] )) {
			return array_intersect_key ( $changes[ 'billing' ], array_flip ( $this->get_attribute_keys () ) );
		}
		return false;
	}

	/**
	 * Returns an array of user attributes.
	 *
	 * @return array
	 */
	private function get_attribute_keys() {
		return [ 'first_name', 'last_name', 'email' 
		];
	}
}
new WC_Stripe_Customer_Manager ();