<?php
/**
 * @package Stripe/Controllers
 * @author PaymentPlugins
 *
 */
class WC_Stripe_Controller_Gateway_Settings extends WC_Stripe_Rest_Controller {

	protected $namespace = 'gateway-settings/';

	public function register_routes() {
		register_rest_route ( $this->rest_uri (), 'apple-domain', [ 
				'methods' => WP_REST_Server::CREATABLE, 
				'callback' => [ $this, 
						'register_apple_domain' 
				], 
				'permission_callback' => array( $this, 
						'admin_permission_check' 
				) 
		] );
		register_rest_route ( $this->rest_uri (), 'create-webook', [ 
				'methods' => WP_REST_Server::CREATABLE, 
				'callback' => [ $this, 'create_webhook' 
				], 
				'permission_callback' => array( $this, 
						'admin_permission_check' 
				) 
		] );
		register_rest_route ( $this->rest_uri (), 'connection-test', [ 
				'methods' => WP_REST_Server::CREATABLE, 
				'callback' => [ $this, 
						'connection_test' 
				], 
				'permission_callback' => array( $this, 
						'admin_permission_check' 
				) 
		] );
	}

	/**
	 * Register the site domain with Stripe for Apple Pay.
	 *
	 * @param WP_REST_Request $request        	
	 */
	public function register_apple_domain($request) {
		$gateway = new WC_Stripe_Gateway ();
		// try to add domain association file.
		if (isset ( $_SERVER[ 'DOCUMENT_ROOT' ] )) {
			$path = $_SERVER[ 'DOCUMENT_ROOT' ] . DIRECTORY_SEPARATOR . '.well-known';
			$file = $path . DIRECTORY_SEPARATOR . 'apple-developer-merchantid-domain-association';
			if (! file_exists ( $file )) {
				require_once ( ABSPATH . '/wp-admin/includes/file.php' );
				if (function_exists ( 'WP_Filesystem' ) && ( WP_Filesystem () )) {
					/**
					 *
					 * @var WP_Filesystem_Base $wp_filesystem
					 */
					global $wp_filesystem;
					if (! $wp_filesystem->is_dir ( $path )) {
						$wp_filesystem->mkdir ( $path );
					}
					$contents = $wp_filesystem->get_contents ( WC_STRIPE_PLUGIN_FILE_PATH . 'apple-developer-merchantid-domain-association' );
					$wp_filesystem->put_contents ( $file, $contents, 0755 );
				}
			}
		}
		$server_name = $_SERVER[ 'SERVER_NAME' ];
		if (strstr ( $server_name, 'www.' )) {
			$server_name_2 = str_replace ( 'www.', '', $server_name );
		} else {
			$server_name_2 = 'www.' . $server_name;
		}
		$domains = [ $server_name, $server_name_2 
		];
		try {
			$failures = 0;
			foreach ( $domains as $domain ) {
				$api_key = wc_stripe_get_secret_key ( 'live' );
				if (empty ( $api_key )) {
					throw new Exception ( __ ( 'You cannot register your domain until you have configured your Live API keys.', 'woo-stripe-payment' ) );
				}
				$result = $gateway->register_domain ( $domain, 'live' );
				if (is_wp_error ( $result )) {
					$failures ++;
					if ($failures > 1) {
						throw new Exception ( $result->get_error_message () );
					}
				}
			}
		} catch ( Exception $e ) {
			return new WP_Error ( 'domain-error', $e->getMessage (), [ 
					'status' => 200 
			] );
		}
		return rest_ensure_response ( [ 
				'message' => sprintf ( __ ( 'Domain registered successfully. You can confirm in your Stripe Dashboard at https://dashboard.stripe.com/account/apple_pay.', 'woo-stripe-payment' ) ) 
		] );
	}

	/**
	 * Create a Stripe webhook for the site.
	 *
	 * @param WP_REST_Request $request        	
	 */
	public function create_webhook($request) {
		$url = wc_stripe ()->rest_api->webhook->rest_url ( 'webhook' );
		$api_settings = wc_stripe ()->api_settings;
		$env = $request->get_param ( 'environment' );
		$gateway = new WC_Stripe_Gateway ( $env );
		// first fetch all webhooks
		$private_key = wc_stripe_get_secret_key ( $env );
		if (empty ( $private_key )) {
			return new WP_Error ( 'webhook-error', __ ( 'You must configure your secret key before creating webhooks.', 'woo-stripe-payment' ), [ 
					'status' => 200 
			] );
		}
		$webhooks = $gateway->webhooks ();
		if (! is_wp_error ( $webhooks )) {
			// validate that the webhook hasn't already been created.
			foreach ( $webhooks->data as $webhook ) {
				if ($webhook[ 'url' ] === $url) {
					return new WP_Error ( 'webhook-error', __ ( 'There is already a webhook configured for this site. If you want to delete the webhook, login to your Stripe Dashboard.', 'woo-stripe-payment' ), [ 
							'status' => 200 
					] );
				}
			}
		}
		
		// create the webhooks and save it to the api settings.
		$webhook = $gateway->create_webhook ( $url, apply_filters ( 'wc_stripe_webhook_events', [ 
				'charge.failed', 'charge.succeeded', 
				'source.chargeable' 
		] ) );
		if (is_wp_error ( $webhook )) {
			return new WP_Error ( $webhook->get_error_code (), $webhook->get_error_message (), [ 
					'status' => 200 
			] );
		} else {
			$api_settings->update_option ( "webhook_url_{$env}", $webhook[ 'url' ] );
			$api_settings->update_option ( "webhook_secret_{$env}", $webhook[ 'secret' ] );
			$api_settings->update_option ( "webhook_id_{$env}", $webhook[ 'id' ] );
			return rest_ensure_response ( [ 
					'message' => sprintf ( __ ( 'Webhook created in Stripe for %s environment. You can test your webhook by logging in to the Stripe dashboard', 'woo-stripe-payment' ), 'live' == $env ? __ ( 'Live', 'woo-stripe-payment' ) : __ ( 'Test', 'woo-stripe-payment' ) ), 
					'secret' => $webhook[ 'secret' ] 
			] );
		}
	}

	/**
	 * Perform a connection test
	 *
	 * @param WP_REST_Request $request        	
	 */
	public function connection_test($request) {
		$mode = $request->get_param ( 'mode' );
		$secret_key = $request->get_param ( 'secret_key' );
		$pub_key = $request->get_param ( 'publishable_key' );
		$gateway = new WC_Stripe_Gateway ( $mode, $secret_key );
		
		try {
			if (empty ( $secret_key )) {
				throw new Exception ( __ ( 'Secret key cannot be empty.', 'woo-stripe-payment' ) );
			}
			if (empty ( $pub_key )) {
				throw new Exception ( __ ( 'Publishable key cannot be empty.', 'woo-stripe-payment' ) );
			}
			// test the secret key
			$response = $gateway->fetch_customers ();
			
			if (is_wp_error ( $response )) {
				throw new Exception ( sprintf ( __ ( 'Mode: %s. Invalid secret key. Please check your entry.', 'woo-stripe-payment' ), $mode ) );
			}
			
			// secret key is good so save it.
			wc_stripe ()->api_settings->update_option ( "secret_key_{$mode}", $secret_key );
			
			// test the publishable key
			$response = wp_remote_post ( 'https://api.stripe.com/v1/payment_methods', [ 
					'headers' => [ 
							'Content-Type' => 'application/x-www-form-urlencoded' 
					], 
					'body' => [ 'key' => $pub_key 
					] 
			] );
			if (is_wp_error ( $response )) {
				throw new Exception ( sprintf ( __ ( 'Mode: %s. Invalid publishable key. Please check your entry.', 'woo-stripe-payment' ), $mode ) );
			}
			if ($response[ 'response' ][ 'code' ] == 401) {
				throw new Exception ( sprintf ( __ ( 'Mode: %s. Invalid publishable key. Please check your entry.', 'woo-stripe-payment' ), $mode ) );
			}
			wc_stripe ()->api_settings->update_option ( "publishable_key_{$mode}", $pub_key );
		} catch ( Exception $e ) {
			return new WP_Error ( 'connection-failure', $e->getMessage (), [ 
					'status' => 200 
			] );
		}
		return rest_ensure_response ( [ 
				'message' => sprintf ( __ ( 'Connection test to Stripe was successful. Mode: %s.', 'woo-stripe-payment' ), $mode ) 
		] );
	}
}